unit FmxUtils;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Consts;

type
  EInvalidDest = class(EStreamError);
  EFCantMove = class(EStreamError);

procedure CopyFile(const FileName, DestName: string);
procedure MoveFile(const FileName, DestName: string);
function GetFileSize(const FileName: string): LongInt;
function FileDateTime(const FileName: string): TDateTime;
function HasAttr(const FileName: string; Attr: Word): Boolean;
function ExecuteFile(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): THandle;
function GetWinVersion(OS: TOSVersionInfo): string;
function GetCpuType(SI: TSystemInfo): string;
function GetTotalMem(MemStat : TMemoryStatus): string;
function GetAvailableMem(MemStat : TMemoryStatus): string;
procedure MakeDir(const FPath: string);
procedure DeleteDir(const FPath: string);

implementation

uses ShellAPI, FileCtrl;

const
  SInvalidDest = 'Destination %s does not exist';
  SFCantMove = 'Cannot move file %s';

procedure CopyFile(const FileName, DestName: TFileName);
var
  CopyBuffer: Pointer; { buffer for copying }
  BytesCopied: Longint;
  Source, Dest: Integer; { handles }
  Destination: TFileName; { holder for expanded destination name }
const
  ChunkSize: Longint = 8192; { copy in 8K chunks }
begin
  Destination := ExpandFileName(DestName); { expand the destination path }
  if HasAttr(Destination, faDirectory) then { if destination is a directory... }
    Destination := Destination + '\' + ExtractFileName(FileName); { ...clone file name }
  GetMem(CopyBuffer, ChunkSize); { allocate the buffer }
  try
    Source := FileOpen(FileName, fmShareDenyWrite); { open source file }
    if Source < 0 then raise EFOpenError.CreateFmt(SFOpenError, [FileName]);
    try
      Dest := FileCreate(Destination); { create output file; overwrite existing }
      if Dest < 0 then raise EFCreateError.CreateFmt(SFCreateError, [Destination]);
      try
        repeat
          BytesCopied := FileRead(Source, CopyBuffer^, ChunkSize); { read chunk }
          if BytesCopied > 0 then { if we read anything... }
            FileWrite(Dest, CopyBuffer^, BytesCopied); { ...write chunk }
        until BytesCopied < ChunkSize; { until we run out of chunks }
      finally
        FileClose(Dest); { close the destination file }
      end;
    finally
      FileClose(Source); { close the source file }
    end;
  finally
    FreeMem(CopyBuffer, ChunkSize); { free the buffer }
  end;
end;


{ MoveFile procedure }
{
  Moves the file passed in FileName to the directory specified in DestDir.
  Tries to just rename the file.  If that fails, try to copy the file and
  delete the original.

  Raises an exception if the source file is read-only, and therefore cannot
  be deleted/moved.
}

procedure MoveFile(const FileName, DestName: string);
var
  Destination: string;
begin
  Destination := ExpandFileName(DestName); { expand the destination path }
  if not RenameFile(FileName, Destination) then { try just renaming }
  begin
    if HasAttr(FileName, faReadOnly) then  { if it's read-only... }
      raise EFCantMove.Create(Format(SFCantMove, [FileName])); { we wouldn't be able to delete it }
      CopyFile(FileName, Destination); { copy it over to destination...}
//      DeleteFile(FileName); { ...and delete the original }
  end;
end;

{ GetFileSize function }
{
  Returns the size of the named file without opening the file.  If the file
  doesn't exist, returns -1.
}

function GetFileSize(const FileName: string): LongInt;
var
  SearchRec: TSearchRec;
begin
  if FindFirst(ExpandFileName(FileName), faAnyFile, SearchRec) = 0 then
    Result := SearchRec.Size
  else Result := -1;
end;

function FileDateTime(const FileName: string): System.TDateTime;
begin
  Result := FileDateToDateTime(FileAge(FileName));
end;

function HasAttr(const FileName: string; Attr: Word): Boolean;
begin
  Result := (FileGetAttr(FileName) and Attr) = Attr;
end;

function ExecuteFile(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): THandle;
var
  zFileName, zParams, zDir: array[0..79] of Char;
begin
  Result := ShellExecute(Application.MainForm.Handle, nil,
    StrPCopy(zFileName, FileName), StrPCopy(zParams, Params),
    StrPCopy(zDir, DefaultDir), ShowCmd);
end;

{Modified on 27/06/98 by Colin LAPLACE. Thanks Andrea Creola}

procedure MakeDir(const FPath: string);
Var Txt,Txt1 : String;
    c1       : Integer;
    flag     : Boolean;

begin
 Txt:=FPath;
 Txt1 := '';
 C1 := 1;
 Flag := True;
 While(Txt<>Txt1) Do
 Begin
   Txt1 := Txt1+Txt[c1];
   if(txt[c1]='\') then
   Begin
(*$i-*)
     if (DirectoryExists(txt1)=False) then
     Begin
       MkDir(txt1);
       Flag := flag and (ioresult=0);
     end;
(*$i+*)
   End;
   c1:=c1+1;
 End;
 if (DirectoryExists(txt1)=False) then
 Begin
   MkDir(txt1);
   Flag := flag and (ioresult=0);
 end;
end;

procedure DeleteDir(const FPath: string);
Var Txt,Txt1 : String;
    c1       : Integer;
    flag     : Boolean;

begin
 Txt:=FPath;
 Txt1 := '';
 C1 := 1;
 Flag := True;
 While(Txt<>Txt1) Do
 Begin
   Txt1 := Txt1+Txt[c1];
   if(txt[c1]='\') then
   Begin
(*$i-*)
     if (DirectoryExists(txt1)=True) then
     Begin
       RmDir(txt1);
       Flag := flag and (ioresult=0);
     end;
(*$i+*)
   End;
   c1:=c1+1;
 End;
 if (DirectoryExists(txt1)=True) then
 Begin
   RmDir(txt1);
   Flag := flag and (ioresult=0);
 end;
end;

function GetWinVersion(OS: TOSVersionInfo): string;
begin
OS.dwOSVersionInfoSize := sizeof(TOSVERSIONINFO);
 GetVersionEx(OS);
 case OS.dwPlatformId of
  VER_PLATFORM_WIN32s        : Result := 'Windows 3.1x/32s';
  VER_PLATFORM_WIN32_WINDOWS : Result := 'Windows 95';
  VER_PLATFORM_WIN32_NT	     : Result := 'Windows NT';
  end;
end;

function GetCpuType(SI: TSystemInfo): string;
begin
 GetSystemInfo(SI);
 case Si.dwProcessorType of
  386 : Result := 'Intel 386';
  486 : Result := 'Intel 486';
  586 : Result := 'Intel Pentium';
 end;
end;

function GetTotalMem(MemStat : TMemoryStatus): string;
begin
MemStat.dwLength := sizeof(TMemoryStatus);
GlobalMemoryStatus(MemStat);
Result:= IntToStr(Trunc(MemStat.dwTotalPhys/1024));
end;

function GetAvailableMem(MemStat : TMemoryStatus): string;
begin
MemStat.dwLength := sizeof(TMemoryStatus);
GlobalMemoryStatus(MemStat);
Result:= IntToStr(Trunc(MemStat.dwAvailPhys/1024));
end;

end.
